extends Node
class_name StateMachine


@export var state_init: NodePath
@onready var state_current: State = get_node("roam")


var item_current: Item = null
var item_focus: Item = null


var cursor_2d_lit: Vector2


@onready var camera: Camera3D = get_parent().get_node("camera/camera_player")
@onready var cursor: Control = get_parent().get_node("cursor")


func _ready() -> void:
	for state in get_children():
		state.state_machine = self
	for item in get_tree().get_nodes_in_group("items"):
		item.state_machine = self
		item.item_collision.mouse_entered.connect( _on_mouse_entered.bind(item) )
		item.item_collision.mouse_exited.connect( _on_mouse_exited.bind(item) )


func _input(event: InputEvent) -> void:
	if event is InputEventMouseMotion:
		cursor_2d_lit = get_viewport().get_mouse_position()
	if event is InputEventMouseButton:
		if Input.is_mouse_button_pressed(MOUSE_BUTTON_LEFT):
			_on_mouse_clicked()
		if Input.is_mouse_button_pressed(MOUSE_BUTTON_RIGHT):
			_on_mouse_unclicked()


func _process(delta: float) -> void:
	var ui_debug: Control = get_node("/root/game/debug")
	if ui_debug != null:
		ui_debug.set_state(state_current.name)
		if item_current == null:
			ui_debug.set_item("none")
			ui_debug.set_interaction("none")
		else:
			ui_debug.set_item(item_current.name)
			if item_current.item_dialogue.has("dialogue"):
				ui_debug.set_interaction(item_current.item_dialogue["dialogue"]["init"]["action"])


func _on_mouse_entered(item: Item) -> void:
	item_focus = item
	state_current.entered( item )


func _on_mouse_exited(item: Item) -> void:
	state_current.exited( item )
	item_focus = null


func _on_mouse_clicked() -> void:
	state_current.clicked()


func _on_mouse_unclicked() -> void:
	state_current.unclicked()


func state_change(target_state: String):
	if has_node(target_state):
		state_current.exit()
		state_current = get_node(target_state)
		state_current.init()
